package com.example.PhidgetSoundSensorExample;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.phidget22.*;

public class SoundSensorExample extends Activity {

	SoundSensor ch;
	SeekBar dataIntervalBar;

	Spinner splRangeSpinner;

	Toast errToast;

	int minDataInterval;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //Hide device information and settings until one is attached
		LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
		settingsAndData.setVisibility(LinearLayout.GONE);

		//set data interval seek bar functionality
		dataIntervalBar = (SeekBar) findViewById(R.id.dataIntervalBar);
		dataIntervalBar.setOnSeekBarChangeListener(new dataIntervalChangeListener());

		//set spl range spinner functionality
		splRangeSpinner = (Spinner) findViewById(R.id.splRangeSpinner);
		splRangeSpinner.setAdapter(new ArrayAdapter<SPLRange>(this,
				android.R.layout.simple_spinner_item, SPLRange.values()));
		splRangeSpinner.setOnItemSelectedListener(new splRangeChangeListener());
		LinearLayout splRangeSection = (LinearLayout) findViewById(R.id.splRangeSection);
		splRangeSection.setVisibility(LinearLayout.GONE);

        try
        {
        	ch = new SoundSensor();

        	//Allow direct USB connection of Phidgets
			if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                com.phidget22.usb.Manager.Initialize(this);

			//Enable server discovery to list remote Phidgets
			this.getSystemService(Context.NSD_SERVICE);
			Net.enableServerDiscovery(ServerType.DEVICE_REMOTE);

			ch.addAttachListener(new AttachListener() {
				public void onAttach(final AttachEvent attachEvent) {
				    AttachEventHandler handler = new AttachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addDetachListener(new DetachListener() {
				public void onDetach(final DetachEvent detachEvent) {
                    DetachEventHandler handler = new DetachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addErrorListener(new ErrorListener() {
				public void onError(final ErrorEvent errorEvent) {
					ErrorEventHandler handler = new ErrorEventHandler(ch, errorEvent);
					runOnUiThread(handler);
				}
			});

			ch.addSPLChangeListener(new SoundSensorSPLChangeListener() {
				public void onSPLChange(SoundSensorSPLChangeEvent splChangeEvent) {
                    SoundSensorSPLChangeEventHandler handler = new SoundSensorSPLChangeEventHandler(ch, splChangeEvent);
                    runOnUiThread(handler);
                }
			});

			ch.open();
        } catch (PhidgetException pe) {
	        pe.printStackTrace();
		}

    }

	private class dataIntervalChangeListener implements SeekBar.OnSeekBarChangeListener {
		@Override
    	public void onProgressChanged(SeekBar seekBar, int progress,
									  boolean fromUser) {
			if(fromUser) {
				try {
					TextView dataIntervalTxt = (TextView) findViewById(R.id.dataIntervalTxt);
					int dataInterval = progress + minDataInterval;
					dataIntervalTxt.setText(String.valueOf(dataInterval));
					ch.setDataInterval(dataInterval);
				} catch (PhidgetException e) {
					e.printStackTrace();
				}
			}
		}

		@Override
		public void onStartTrackingTouch(SeekBar seekBar) {}

		@Override
		public void onStopTrackingTouch(SeekBar seekBar) {}
	}

	private class splRangeChangeListener implements Spinner.OnItemSelectedListener {
		@Override
		public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {
			try {
				ch.setSPLRange(SPLRange.valueOf(parentView.getItemAtPosition(position).toString()));
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		@Override
		public void onNothingSelected(AdapterView<?> parentView) {
			
		}

	}

	class AttachEventHandler implements Runnable {
    	Phidget ch;

		public AttachEventHandler(Phidget ch) {
			this.ch = ch;
		}

		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
			settingsAndData.setVisibility(LinearLayout.VISIBLE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);

			//Clear the error toast if a new ch is attached
			if (errToast != null) {
				errToast.cancel();
			}

			attachedTxt.setText("Attached");
			try {
				TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
				TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
				TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
				TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
				TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
				TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

				nameTxt.setText(ch.getDeviceName());
				serialTxt.setText(Integer.toString(ch.getDeviceSerialNumber()));
				versionTxt.setText(Integer.toString(ch.getDeviceVersion()));
				channelTxt.setText(Integer.toString(ch.getChannel()));
				hubPortTxt.setText(Integer.toString(ch.getHubPort()));
				labelTxt.setText(ch.getDeviceLabel());

				TextView dataIntervalTxt = (TextView) findViewById(R.id.dataIntervalTxt);
				dataIntervalTxt.setText(String.valueOf(((SoundSensor)ch).getDataInterval()));

				minDataInterval = ((SoundSensor)ch).getMinDataInterval();

				SeekBar dataIntervalBar = (SeekBar) findViewById(R.id.dataIntervalBar);
				dataIntervalBar.setProgress(((SoundSensor)ch).getDataInterval() - minDataInterval);

				//Limit the maximum dataInterval on the SeekBar to 5000 so it remains usable
				if(((SoundSensor)ch).getMaxDataInterval() >= 5000)
                    dataIntervalBar.setMax(5000 - minDataInterval);
				else
                    dataIntervalBar.setMax(((SoundSensor)ch).getMaxDataInterval() - minDataInterval);

				LinearLayout splRangeSection = (LinearLayout) findViewById(R.id.splRangeSection);
				splRangeSection.setVisibility(LinearLayout.VISIBLE);

				//Set the default selection to 102dB
				splRangeSpinner.setSelection(0);

			} catch (PhidgetException e) {
				e.printStackTrace();
			}

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

    class DetachEventHandler implements Runnable {
    	Phidget ch;
    	
    	public DetachEventHandler(Phidget ch) {
    		this.ch = ch;
    	}
    	
		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);

			settingsAndData.setVisibility(LinearLayout.GONE);
			LinearLayout splRangeSection = (LinearLayout) findViewById(R.id.splRangeSection);
			splRangeSection.setVisibility(LinearLayout.GONE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);
			attachedTxt.setText("Detached");

			TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
			TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
			TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
			TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
			TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
			TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

			nameTxt.setText(R.string.unknown_val);
			serialTxt.setText(R.string.unknown_val);
			versionTxt.setText(R.string.unknown_val);
			channelTxt.setText(R.string.unknown_val);
			hubPortTxt.setText(R.string.unknown_val);
			labelTxt.setText(R.string.unknown_val);

			//clear data on detach
			((TextView)findViewById(R.id.dBTxt)).setText("");
			((TextView)findViewById(R.id.dBATxt)).setText("");
			((TextView)findViewById(R.id.dBCTxt)).setText("");

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

    class ErrorEventHandler implements Runnable {
        Phidget ch;
        ErrorEvent errorEvent;

        public ErrorEventHandler(Phidget ch, ErrorEvent errorEvent) {
            this.ch = ch;
            this.errorEvent = errorEvent;
        }

        public void run() {
			 if (errToast == null)
				 errToast = Toast.makeText(getApplicationContext(), errorEvent.getDescription(), Toast.LENGTH_SHORT);

			 //replace the previous toast message if a new error occurs
			 errToast.setText(errorEvent.getDescription());
			 errToast.show();
        }
    }

	class SoundSensorSPLChangeEventHandler implements Runnable {
		Phidget ch;
		SoundSensorSPLChangeEvent splChangeEvent;

		public SoundSensorSPLChangeEventHandler(Phidget ch, SoundSensorSPLChangeEvent splChangeEvent) {
			this.ch = ch;
			this.splChangeEvent = splChangeEvent;
		}

		public void run() {
			TextView dBTxt = (TextView)findViewById(R.id.dBTxt);
		    TextView dBATxt = (TextView)findViewById(R.id.dBATxt);
			TextView dBCTxt = (TextView)findViewById(R.id.dBCTxt);

			dBTxt.setText(String.valueOf(splChangeEvent.getDB()));
			dBATxt.setText(String.valueOf(splChangeEvent.getDBA()));
			dBCTxt.setText(String.valueOf(splChangeEvent.getDBC()));
		}
	}

    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	try {
			ch.close();

		} catch (PhidgetException e) {
			e.printStackTrace();
		}

		//Disable USB connection to Phidgets
    	if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
            com.phidget22.usb.Manager.Uninitialize();
    }

}

